/*
 *  Revision 20181206, Kai Peter
 *  - compare length of VM name from reg file with argv*
 *  Revision 20180410, Kai Peter
 *  - switched to 'qstrings'
 *  Revision 20180318, Kai Peter
 *  - added 'flagquiet', code cleanup
 *  Revision 20180306, Kai Peter
 *  - initial new file
*/
#include <unistd.h>
#include "socket.h"
#include "buffer.h"
#include "errmsg.h"
#include "getln.h"
#include "getoptb.h"
#include "ip.h"
#include "iopause.h"
#include "open.h"
#include "qstrings.h"

#define WHO "qemu-cmd"

uint16 local_port = 0;
uint16 remote_port;

char local_ip[16] = {0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0};
char remote_ip[16] = {0}; //{0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,1};

uint32 netif = 0;
int ipv4socket;
unsigned int timeout = 26;

char inbuf[128];
char outbuf[1024];
buffer b;

void usage(void) { errlog(0,ERROR,B("usage: ",WHO," -c <cmd> VMID")); }

int main(int argc, char **argv) {
  int opt;
  int s;
  char *fn, *tmp, *vmfn;
  unsigned int pos;
  int fd, match;
  stralloc line = {0};
  stralloc cmd = {0};
  ssize_t n;
  int flagquiet = 0;

  while ((opt = getopt(argc,argv,"4c:qt:v")) != opteof)
  switch(opt) {
	case '4': ipv4socket = 1; break;   /* force IPv4 */
//	case '6': ipv4socket = 0; break;
    case 'c': stralloc_copys(&cmd,optarg);
  			  stralloc_cats(&cmd,"\r\n");
  			  stralloc_0(&cmd);
  			  break;
//	case 'p': scan_ulong(optarg,(unsigned long *)&remote_port); break;
	case 'q': flagquiet = 1; break;
    case 't': scan_ulong(optarg,(unsigned long *)&timeout); break;
    case 'v': loglevel = DEBUG; break;
    default: usage();
  }
  argc -= optind;
  argv += optind;

  if (!*argv) usage();

  if (flagquiet) buffer_2->fd = -1;

  fn = "/etc/qemu/qemu-init.reg";
  fd = open_read(fn);
  if (fd == -1) errint(errno,B("Couldn't open file: ",fn));

  buffer_init(&b,read,fd,inbuf,sizeof(inbuf));

  /* get the location of the vm from config file qemu-init.reg */
  while (1) {
	if (getln(&b,&line,&match,'\n') != 0)
	  errint(EHARD,B("Couldn't read file: ",fn));
	if (!match && !line.len) { break; } /* EOF */

    pos = byte_chr(line.s,str_len(line.s),':');  /* get position of first ':' */
    /* stralloc_copys(&line,strsplit(line.s,pos,LPART));  !? <- readability ? */
    tmp = strsplit(line.s,pos,QLEFT);               /* readability over speed */
    if (!stralloc_copys(&line,tmp)) errmem;      /*           "            */
    stralloc_0(&line);
	strcpys(&vmfn,line.s);     /* save content of the first field temp. for later usage */

    pos = byte_rchr(line.s,str_len(line.s),'/'); /* get position of last '/' in field 1 */
    tmp = strsplit(line.s,pos,QRIGHT);
    if (!stralloc_copys(&line,tmp)); errmem;
//    stralloc_0(&line);

	pos = byte_chr(line.s,str_len(line.s),'-');
    tmp = strsplit(line.s,pos,QLEFT);
    if (!stralloc_copys(&line,tmp)) errmem;
//    stralloc_0(&line);

    if(str_len(*argv) == str_len(tmp))  /* length's have to match */
      if (str_equal(*argv,line.s)) break;
  }
  close(fd);

  /* error on invalid param */
  if (!line.len)
    errint(ESOFT,B("Couldn't find registered VM ",*argv));

  pos = byte_rchr(vmfn,str_len(vmfn),'.');
  vmfn = strsplit(vmfn,pos,QLEFT);    /* cut the file ending '.rc' */
  strcats(&vmfn,".pid");         /* now line contains the pid file */

  fn = vmfn;    /* vmfn contains the 'pid' file with absolute path */
  stralloc_init(&line);    /* initialize before use again */
  fd = open_read(fn);
  if (fd == -1) errint(errno,B("Couldn't open file: ",fn));
  while (1) {
	if (getln(&b,&line,&match,'\n') != 0)
	  errint(EHARD,B("Couldn't read file: ",fn));
	if (!match && !line.len) { break; } /* EOF */
	if (str_pcmp(line.s,"MON=")) break;
  }
  close(fd);
  /* error getting the port */
  if (!line.len)
    errint(ESOFT,B("Couldn't find port for registerd VM ",*argv));

  /* get the port from 'MON=' line from 'pid' file */
  pos = byte_chr(line.s,str_len(line.s),'=');
  stralloc_copys(&line,strsplit(line.s,pos,QRIGHT));
//  stralloc_0(&line);
  scan_ulong(line.s,(unsigned long *)&remote_port);
  remote_port = remote_port + 100;   /* add 100 per definition */

  /* */
  if((s = socket_tcp()) == -1)
    errint(errno,"unable to create local socket");

  if (!ipv4socket) ipv4socket = ip6_isv4mapped(local_ip);

  /* timeoutconn does the 'socket_connect' */
  if(timeoutconn(s,remote_ip,remote_port,timeout,netif) != 0)
	errint(errno,"could not connect to socket");

  /* write data to socket */
  buffer_init(&b,(ssize_t (*)())timeoutwrite,s,outbuf,sizeof(outbuf));
  buffer_put(&b,cmd.s,cmd.len);
  buffer_flush(&b);   /* we don't need '\r\n' here */

  /* read data from socket */
  buffer_init(&b,(ssize_t (*)())timeoutread,s,inbuf,sizeof(inbuf));
  stralloc_init(&line);             /* we can re-use this stralloc here */
  for (;;) {
    n = timeoutread(s,inbuf,sizeof(inbuf),0);     /* timeout is 0 here! */
	if (n < 0) { break;	}
    stralloc_catb(&line,inbuf,n);  /* n is the number of received bytes */
  }

  stralloc_0(&line);
  buffer_puts(buffer_1,line.s);
  buffer_putsflush(buffer_1,"\r");

  close(s);
  _exit(0);
}
