#********************************************************************************
# Handle vnc ports configuration and vnc connections                            *
#                                                                               *
#  Author: Kai Peter, ©2019-                                                    *
# Version: 0.27                                                                 *
# License: ISC                                                                  *
#                                                                               *
# Description: qemu-vnc has two modes:                                          *
#              1. user mode: connect to qemu guest <name> via vnc (default)     *
#              2. superuser mode: get all qemu guests which have vnc enabled at *
#                 start time (via qemu-adm/qemud) and write result into file.   *
#********************************************************************************
VERSION=`cat $0 | grep -m 1 'Version' | cut -d\   -f3`       # get version number
ME=`basename $0`

declare -i DBG=0

QEMUHOST=`hostname`
FN="VNCFILE"
# create the temp. folder if not exists as user 'qemud'
TMP=`dirname "$FN"`
[ ! -d "$TMP" ] && sudo -u QUSER mkdir -p -m 0775 "$TMP"

usage() { echo "Usage: $ME [ -h | -V ] | <name>" ; exit 0 ; }

#********************************************************************************
getAll() {      # create file with vnc enabled guests - usually done by qemu-init
  if [ "$(id -u)" -ne 0 ] ; then
    getent group QGROUP | grep "$USER" >/dev/null && EC=$?
    [ "$EC" -ne 0 ] && echo "$ME: privilege escalation required!" && exit $EC
  fi

  local P    # port
  local N    # name
  local A    # awk result
  [ -f "$FN" ] && rm "$FN"

  for N in `sudo qemud status | grep 'is running' | cut -d\  -f1`
  do
    TMP=`ps ax | grep "$N" | grep qemu-system | sed 's/ -/;/g'`
    A=`awk -F";" '{for (i=2;i<=NF;i++) printf("\t-%s\n", $i)}' <<< "$TMP"`

    P=`echo "$A" | grep 'vnc' | cut -d: -f2`  # get the vnc port from qemu
    if [ -n "$P" ] ; then
      P=`expr 5900 + $P`                      # calculate the 'real' port
      # write <name>, host and vnc port to VNCFILE
      echo `echo "$N:$QEMUHOST" | tr '[A-Z]' '[a-z]'`":$P" >> "$FN"
    fi
  done
  # the next is explicit granted by sudoer's config
  sudo chmod 0666 "$FN"
  sudo chown QUSER.QGROUP "$FN"
}

# standard options
( [ "$1" = "-h" ] || [ "$1" = "--help" ] ) && usage
( [ "$1" = "-V" ] || [ "$1" = "--version" ] ) && echo "$ME-$VERSION" && exit 0

# create file "VNCFILE":
[ ! "$1" ] && getAll && exit $?    # w/o a parameter and exit
[ ! -f "$FN" ] && getAll           # if not exists


[ "$1" ] && NAME=`echo "$1" | tr '[A-Z]' '[a-z]'`
# initiate a vnc connection
[ -n "$NAME" ] && PORT=`grep "$NAME" "$FN" | cut -d: -f3` && [ -n "$PORT" ] && \
  vncviewer "$QEMUHOST":"$PORT"

exit $?
