#********************************************************************************
# qemud - simple wrapper script to start multiple qemu virtual machines         *
#                                                                               *
#  Author: Kai Peter, ©2016-                                                    *
# Version: 0.37                                                                 *
# License: ISC                                                                  *
#                                                                               *
# Description: qemud is a wrapper to handle links of qemu-init. Commonly it is  *
#              a link in '/etc/init.d' (recommended).                           *
#********************************************************************************
VERSION=`cat $0 | grep -m 1 'Version' | cut -d\   -f3`       # get version number
ME=`basename $0`                                             # set program name

usage() { echo "Usage: "`basename $0`" start|stop|status" ; exit 1 ; }

[ $1 ] || usage
[ $2 ] && usage
( [ $1 = "-V" ] || [ $1 = "--version" ] ) && echo "$ME-$VERSION" && exit 0
[ $1 = "start" ] || [ $1 = "stop" ] || [ $1 = "status" ] || usage

autostart() {
  [ "$1" = "start" ] || return
  AUTOSTART=`echo "$AUTOSTART" | tr '[A-Z]' '[a-z]'`
  [ "$AUTOSTART" = "y"   ] && AUTOSTART=1
  [ "$AUTOSTART" = "on"  ] && AUTOSTART=1
  [ "$AUTOSTART" = "yes" ] && AUTOSTART=1
  [ "$AUTOSTART" = "1" ] || VM=""
}

CFF="REGFILE"
[ ! -f "$CFF" ] && echo "Config file '$CFF' not found! Aborting!" && exit 4

export QEMUINIT="$ME"               # required to control behavior of 'qemu-init'
while IFS= read L ; do
  VM=$(echo "$L" | grep -v '^#' | cut -d: -f1)
  AUTOSTART=$(echo "$L" | cut -d: -f2)
  [ "$AUTOSTART" ] && autostart "$1"
  # Never call 'qemu-adm' as a background job here!  'qemu-init' puts itself into
  # background if required (e.g. if the $BOOTDELAY option is set)
  if [ -n "$VM" ] && [ -f "$VM" ] ; then qemu-adm "--$1" "$VM" ; fi
done < "$CFF"
